#include <ESP8266WiFi.h>
#include <Time.h>
#include <TimeLib.h>

const char* ssid = "SSID";
const char* password = "WACHTWOORD";

const int rood = 5; // D1
const int groen = 4; // D2
const int blauw = 0; // D3
const int wit = 14; // D5
const int ingebouwde_led = 2; // D4
const char* timehost = "arduino.reshift.nl";
const String url="/wake-up/tijd.php";
const String reeks_url="/wake-up/get_values.php";
const int httpPort = 80;
const unsigned long interval = 900000; // Tijd (in milliseconden) tussen het synchroniseren van de tijd. 
unsigned long previousMillis = 0; 
bool eerstesync = true; 
int ma_uur;
int ma_min;
int di_uur;
int di_min; 
int wo_uur; 
int wo_min;
int do_uur;
int do_min;
int vr_uur;
int vr_min;
int za_uur;
int za_min;
int zo_uur;
int zo_min;

WiFiClient client;

void setup() {
  pinMode(rood, OUTPUT);
  pinMode(groen, OUTPUT);
  pinMode(blauw, OUTPUT);
  pinMode(wit, OUTPUT);
  pinMode(ingebouwde_led, OUTPUT); 
  Serial.begin(115200);
  digitalWrite(rood, LOW);
  digitalWrite(groen, LOW);
  digitalWrite(blauw, LOW);
  digitalWrite(wit, LOW); 
  digitalWrite(ingebouwde_led, LOW); // Led aan. 

  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());
}

int lees_tot_tag(String begin_tag, String einde_tag, String inhoud) {

  int datum;
  int begin_pos = inhoud.indexOf(begin_tag);
  if(begin_pos != -1) {
    int einde_pos = inhoud.indexOf(einde_tag);
    String result = inhoud.substring(begin_pos + begin_tag.length(), einde_pos);
    Serial.println(result);
    datum = result.toInt();
  }
  return datum;
}

String getValue(String data, char separator, int index)
{
    int found = 0;
    int strIndex[] = { 0, -1 };
    int maxIndex = data.length() - 1;
    for (int i = 0; i <= maxIndex && found <= index; i++) {
        if (data.charAt(i) == separator || i == maxIndex) {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}

void getParams() {

  unsigned long currentMillis = millis();
  if ((unsigned long)(currentMillis - previousMillis) >= interval || eerstesync == true) {

    Serial.print("Verbind met ");
    Serial.println(timehost);

    if (!client.connect(timehost, httpPort)) {
      Serial.println("Verbinding mislukt");
      client.stop();
      return;
    }

    Serial.print("Vraag URL op: ");
    Serial.println(reeks_url);
    client.print(String("GET ") + reeks_url + " HTTP/1.1\r\n" +
                   "Host: " + timehost + "\r\n\r\n");
               
    unsigned long timeout = millis();
    while (client.available() == 0) {
      if (millis() - timeout > 5000) {
        Serial.println("Verbinding mislukt: timeout");
        client.stop();
        return;
      }
    }

    while(client.available()){
      String page = client.readString();
      String selection = getValue(page, '#', 1);
      String ma_u = getValue(selection, '\n', 1);
      String ma_m = getValue(selection, '\n', 2);
      String di_u = getValue(selection, '\n', 3);
      String di_m = getValue(selection, '\n', 4);
      String wo_u = getValue(selection, '\n', 5);
      String wo_m = getValue(selection, '\n', 6);
      String do_u = getValue(selection, '\n', 7);
      String do_m = getValue(selection, '\n', 8);
      String vr_u = getValue(selection, '\n', 9);
      String vr_m = getValue(selection, '\n', 10);
      String za_u = getValue(selection, '\n', 11);
      String za_m = getValue(selection, '\n', 12);
      String zo_u = getValue(selection, '\n', 13);
      String zo_m = getValue(selection, '\n', 14);
     
      ma_uur = ma_u.toInt();
      ma_min = ma_m.toInt();
      di_uur = di_u.toInt();
      di_min = di_m.toInt();
      wo_uur = wo_u.toInt();
      wo_min = wo_m.toInt();
      do_uur = do_u.toInt();
      do_min = do_m.toInt();
      vr_uur = vr_u.toInt();
      vr_min = vr_m.toInt();
      za_uur = za_u.toInt();
      za_min = za_m.toInt();
      zo_uur = zo_u.toInt();
      zo_min = zo_m.toInt();
    }

    Serial.println("Verbreek verbinding");
    client.stop();

    Serial.print("Verbind met ");
    Serial.println(timehost);

    if (!client.connect(timehost, httpPort)) {
      Serial.println("Verbinding mislukt");
      client.stop();
      return;
    }

    Serial.print("Vraag URL op: ");
    Serial.println(url);
    client.print(String("GET ") + url + " HTTP/1.1\r\n" +
                   "Host: " + timehost + "\r\n\r\n");
               
    unsigned long timeout2 = millis();
    while (client.available() == 0) {
      if (millis() - timeout2 > 5000) {
        Serial.println("Verbinding mislukt: timeout");
        client.stop();
        return;
      }
    }

    while(client.available()){

      String page = client.readString();
      String selection = getValue(page, '#', 1);
      String jaar = getValue(selection, '\n', 1);
      String maand = getValue(selection, '\n', 2);
      String dag = getValue(selection, '\n', 3);
      String uur = getValue(selection, '\n', 4);
      String minuut = getValue(selection, '\n', 5);
      String seconde = getValue(selection, '\n', 6);
    
      int datum_jaar = jaar.toInt();
      int datum_maand = maand.toInt();
      int datum_dag = dag.toInt();
      int datum_uur = uur.toInt();
      int datum_minuut = minuut.toInt();
      int datum_seconde = seconde.toInt();

      setTime(datum_uur,datum_minuut,datum_seconde,datum_dag,datum_maand,datum_jaar);
      Serial.println("Verbreek verbinding");
      client.stop();
    }
    previousMillis = currentMillis;
    eerstesync = false;
  }
}

void sunrise() {
  Serial.println("Gestart");
  int i = 0;
  for (int j = 1024; j >= 1; j--) {
    Serial.print(j*3.5);
    Serial.print(" ");
    Serial.println(i);
    analogWrite(wit, i);
    analogWrite(rood, i);
    analogWrite(groen, i);
    analogWrite(blauw, i);
    i++;
    delay(j*3.5);
  }
  Serial.println("Gestopt");
  for(int k = 0; k <= 1800; k++) {
    delay(1000);
  }
  digitalWrite(wit, LOW);
  digitalWrite(rood, LOW);
  digitalWrite(groen, LOW);
  digitalWrite(blauw, LOW);
}

void loop() {
  getParams();

  if(timeStatus() == timeNeedsSync) {
    digitalWrite(ingebouwde_led, LOW); // Led aan. 
  }
  else if(timeStatus() == timeSet) {
    digitalWrite(ingebouwde_led, HIGH); // Led uit. 
  }

  Serial.print("Dag ");
  Serial.println(weekday());
  Serial.print("Uur: ");
  Serial.println(hour());
  Serial.print("Minuut: ");
  Serial.println(minute());

  Serial.println(ma_uur);
  Serial.println(ma_min);
  Serial.println(di_uur);
  Serial.println(di_min);
  Serial.println(wo_uur);
  Serial.println(wo_min);
  Serial.println(do_uur);
  Serial.println(do_min);
  Serial.println(vr_uur);
  Serial.println(vr_min);
  Serial.println(za_uur);
  Serial.println(za_min);
  Serial.println(zo_uur);
  Serial.println(zo_min);

  if((weekday() == 2 && hour() == ma_uur && minute() == ma_min) || (weekday() == 3 && hour() == di_uur && minute() == di_min) || (weekday() == 4 && hour() == wo_uur && minute() == wo_min) || (weekday() == 5 && hour() == do_uur && minute() == do_min) || (weekday() == 6 && hour() == vr_uur && minute() == vr_min) || (weekday() == 7 && hour() == za_uur && minute() == za_min) || (weekday() == 1 && hour() == zo_uur && minute() == zo_min)) {
    sunrise();
  }

  delay(10000);
}
